/*******************************************************************************//**
 * \file sng_tc_if.h
 * \brief Sangoma Transcoding library for Vocallo/Sangoma Transcoding Board
 * \brief Provides User a Unified/OS Agnostic API to Sangoma Transcoding Board
 *
 * Author(s):   Nenad Corbic <ncorbic@sangoma.com>
 *
 * Copyright:	(c) 2009-2009 Nenad Corbic <ncorbic@sangoma.com>
*/


#ifndef __LIB_SANGOMA_TRANSCODE_IF__
#define __LIB_SANGOMA_TRANSCODE_IF__

enum sngtc_codec_definition {
	SNGTC_CODEC_MIN_ID	= 0,

	SNGTC_CODEC_PCMU	= 1,
	SNGTC_CODEC_PCMA	= 2,
	SNGTC_CODEC_L16_1	= 3,
	SNGTC_CODEC_L16_2	= 4,
	SNGTC_CODEC_G729AB	= 5,
	SNGTC_CODEC_G722	= 6,
	SNGTC_CODEC_GSM_FR	= 7,
	SNGTC_CODEC_GSM_EFR	= 8,
	SNGTC_CODEC_AMR		= 9,
	SNGTC_CODEC_AMR_WB	= 10,
	SNGTC_CODEC_G726_16	= 11,
	SNGTC_CODEC_G726_24	= 12,
	SNGTC_CODEC_G726_32	= 13,
	SNGTC_CODEC_G726_40	= 14,
	SNGTC_CODEC_ILBC	= 15,

	SNGTC_CODEC_MAX_ID,
};



#define SNGTC_CODEC_ID_DECODE(id) \
		(id==SNGTC_CODEC_PCMU) ? "SNGTC_CODEC_PCMU" : \
		(id==SNGTC_CODEC_PCMA) ? "SNGTC_CODEC_PCMA" : \
		(id==SNGTC_CODEC_L16_1) ? "SNGTC_CODEC_L16_1" : \
		(id==SNGTC_CODEC_L16_2) ? "SNGTC_CODEC_L16_2" : \
		(id==SNGTC_CODEC_G729AB) ? "SNGTC_CODEC_G729AB" : \
		(id==SNGTC_CODEC_G722) ? "SNGTC_CODEC_G722" : \
		(id==SNGTC_CODEC_GSM_FR) ? "SNGTC_CODEC_GSM_FR" : \
		(id==SNGTC_CODEC_GSM_EFR) ? "SNGTC_CODEC_GSM_EFR" : \
		(id==SNGTC_CODEC_AMR) ? "SNGTC_CODEC_AMR" : \
		(id==SNGTC_CODEC_G726_32) ? "SNGTC_CODEC_G726_32" : \
		(id==SNGTC_CODEC_ILBC) ? "SNGTC_CODEC_ILIBC" : "Unknown"


enum sngtc_error_codes {
	SNTC_OK,
	SNTC_ERROR,
	SNTC_ERROR_MEM_ALLOC,
};


typedef struct sngtc_codec_request_leg
{
	uint32_t 	codec_id;
	uint32_t	ms;

	uint32_t	host_ip;
	uint32_t	host_netmask;
	uint32_t	host_udp_port;

} sngtc_codec_request_leg_t;


typedef struct sngtc_codec_request
{
	void 		*usr_priv;
	uint64_t	tag;

	uint8_t		rtcp_enable;

	struct sngtc_codec_request_leg a;
	struct sngtc_codec_request_leg b;

} sngtc_codec_request_t;



typedef struct sngtc_codec_reply_leg
{
	uint32_t	codec_ip;
	uint32_t	codec_netmask;
	uint32_t	codec_udp_port;

	uint32_t	host_ip;
	uint32_t	host_netmask;
	uint32_t	host_udp_port;

	uint32_t	iana_code;

}sngtc_codec_reply_leg_t;


typedef struct sngtc_codec_reply
{
	uint32_t 	codec_module_session_idx;
	uint32_t	codec_rtp_session_idx;

	struct sngtc_codec_reply_leg a;
	struct sngtc_codec_reply_leg b;

	void		*tx_fd;
	uint32_t	tx_iana;

	void		*rx_fd;
	uint32_t	rx_iana;

} sngtc_codec_reply_t;



#define SNGTC_MAX_MAC_RANGE 		32
#define SNGTC_MAX_HOST_VOCALLO_NIC 	32
#define SNGTC_MAX_ETH_SZ			6

typedef struct sngtc_eth_mac
{
	char mac_str[18];
	uint8_t mac[6];
}sngtc_eth_mac_t;

typedef struct sngtc_ip
{
	char ip_str[25];
	uint32_t ip;
}sngtc_ip_t;



/*!
  \struct sngtc_host_nic_vocallo_cfg
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
  \typedef sngtc_host_vocallo_cfg
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
*/
typedef struct sngtc_host_nic_vocallo_cfg
{
	uint32_t 	vocallo_base_udp_port;
	uint32_t	vocallo_ip;
	uint32_t	vocallo_ip_netmask;

	uint32_t	host_ip;
	uint32_t	host_ip_netmask;

	sngtc_eth_mac_t	host_mac;

	/* Optional to select a speicifc vocallo */
	sngtc_eth_mac_t	vocallo_macs[SNGTC_MAX_MAC_RANGE];
	uint32_t		vocallo_macs_sz;

	uint8_t		silence_suppression;

} sngtc_host_nic_vocallo_cfg_t;



/*!
  \struct sngtc_init_cfg
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
  \typedef sngtc_init_cfg_t
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
*/ 

typedef struct sngtc_init_cfg 
{
	/* Library operation mode: LIBRARY, SOAP_CLIENT, SOAP_SERVER */
	int								operation_mode;

	/* Host NIC configuration is only used in non soap_mode */
	uint32_t						host_nic_vocallo_sz;
	sngtc_host_nic_vocallo_cfg_t	host_nic_vocallo_cfg[SNGTC_MAX_HOST_VOCALLO_NIC];

	int (*log)( int level, char *fmt, ...);
	int (*create_rtp)(void * usr_priv, sngtc_codec_request_leg_t *codec_reg_leg, sngtc_codec_reply_leg_t* codec_reply_leg, void **rtp_fd);
	int (*create_rtp_port)(void * usr_priv, uint32_t host_ip, uint32_t *rtp_port,  void **rtp_fd);
	int (*destroy_rtp)(void * usr_priv, void *fd);

} sngtc_init_cfg_t;

int sngtc_detect_init_modules(sngtc_init_cfg_t *cfg, int *detected);
int sngtc_activate_modules(sngtc_init_cfg_t *cfg, int *activated);
int sngtc_deactivate_modules(void);

int sngtc_create_transcoding_session (struct sngtc_codec_request *codec_req, struct sngtc_codec_reply *codec_reply, int start_module);
int sngtc_free_transcoding_session   (struct sngtc_codec_reply *codec_reply);

int sngtc_detected_vocallo_modules(void);
char *sngtc_module_mac(uint16_t module_idx);

#endif

